@@ -29,6 +29,7 @@
 #include <string.h>
 #include <getopt.h>
 #include <limits.h>
+#include <unistd.h>
 
 #include <vector>
 #include <string>
@@ -41,10 +42,10 @@
 #endif
 
 #include <ttfautohint.h>
+#include <numberset.h>
 
 
 #ifdef _WIN32
-#  include <unistd.h>
 #  include <fcntl.h>
 #  define SET_BINARY(f) do { \
                           if (!isatty(fileno(f))) \
@@ -171,7 +172,7 @@ show_help(bool
 "  -i, --ignore-restrictions  override font license restrictions\n"
 "  -l, --hinting-range-min=N  the minimum PPEM value for hint sets\n"
 "                             (default: %d)\n"
-"  -n  --no-info              don't add ttfautohint info\n"
+"  -n, --no-info              don't add ttfautohint info\n"
 "                             to the version string(s) in the `name' table\n"
 "  -p, --pre-hinting          apply original hints in advance\n",
           TA_HINTING_LIMIT, TA_HINTING_RANGE_MIN);
@@ -186,12 +187,16 @@ show_help(bool
 "                             with possible values `g' for grayscale,\n"
 "                             `G' for GDI ClearType, and `D' for\n"
 "                             DirectWrite ClearType (default: G)\n"
+"  -W, --windows-compatibility\n"
+"                             add blue zones for `usWinAscent' and\n"
+"                             `usWinDescent' to avoid clipping\n"
 "  -x, --increase-x-height=N  increase x height for sizes in the range\n"
 "                             6<=PPEM<=N; value 0 switches off this feature\n"
 "                             (default: %d)\n"
 "  -X, --x-height-snapping-exceptions=STRING\n"
 "                             specify a comma-separated list of\n"
-"                             x-height snapping exceptions\n"
+"                             x-height snapping exceptions, for example\n"
+"                             \"-9, 13-17, 19\" (default: \"\")\n"
 "\n",
           TA_HINTING_RANGE_MAX, TA_INCREASE_X_HEIGHT);
 
@@ -316,12 +321,16 @@ main(int argc,
   bool dw_cleartype_strong_stem_width = false;
 
   bool ignore_restrictions = false;
+  bool windows_compatibility = false;
   bool pre_hinting = false;
   bool hint_with_components = true;
   bool no_info = false;
   int latin_fallback = 0; // leave it as int; this probably gets extended
   bool symbol = false;
 
+  const char* x_height_snapping_exceptions_string = NULL;
+  bool have_x_height_snapping_exceptions_string = false;
+
 #ifndef BUILD_GUI
   bool debug = false;
 
@@ -371,6 +380,7 @@ main(int argc,
       {"symbol", no_argument, NULL, 's'},
       {"verbose", no_argument, NULL, 'v'},
       {"version", no_argument, NULL, 'V'},
+      {"windows-compatibility", no_argument, NULL, 'W'},
       {"x-height-snapping-exceptions", required_argument, NULL, 'X'},
 
       // Qt options
@@ -403,7 +413,7 @@ main(int argc,
     };
 
     int option_index;
-    int c = getopt_long_only(argc, argv, "cfG:hil:npr:stVvx:X:w:",
+    int c = getopt_long_only(argc, argv, "cfG:hil:npr:stVvw:Wx:X:",
                              long_options, &option_index);
     if (c == -1)
       break;
@@ -473,15 +483,18 @@ main(int argc,
       dw_cleartype_strong_stem_width = strchr(optarg, 'D') ? true : false;
       break;
 
+    case 'W':
+      windows_compatibility = true;
+      break;
+
     case 'x':
       increase_x_height = atoi(optarg);
       have_increase_x_height = true;
       break;
 
     case 'X':
-#ifdef CONSOLE_OUTPUT
-      fprintf(stderr, "Option `-x' not implemented yet\n");
-#endif
+      x_height_snapping_exceptions_string = optarg;
+      have_x_height_snapping_exceptions_string = true;
       break;
 
 #ifndef BUILD_GUI
@@ -524,9 +537,14 @@ main(int argc,
     hinting_limit = TA_HINTING_LIMIT;
   if (!have_increase_x_height)
     increase_x_height = TA_INCREASE_X_HEIGHT;
+  if (!have_x_height_snapping_exceptions_string)
+    x_height_snapping_exceptions_string = "";
 
 #ifndef BUILD_GUI
 
+  if (!isatty(fileno(stderr)) && !debug)
+    setvbuf(stderr, (char*)NULL, _IONBF, BUFSIZ);
+
   if (hinting_range_min < 2)
   {
     fprintf(stderr, "The hinting range minimum must be at least 2\n");
@@ -553,6 +571,42 @@ main(int argc,
     exit(EXIT_FAILURE);
   }
 
+  number_range* x_height_snapping_exceptions = NULL;
+
+  if (have_x_height_snapping_exceptions_string)
+  {
+    const char* s;
+
+
+    s = number_set_parse(x_height_snapping_exceptions_string,
+                         &x_height_snapping_exceptions,
+                         6, 0x7FFF);
+    if (*s)
+    {
+      if (x_height_snapping_exceptions == NUMBERSET_ALLOCATION_ERROR)
+        fprintf(stderr, "Allocation error while scanning"
+                        " x height snapping exceptions\n");
+      else {
+        if (x_height_snapping_exceptions == NUMBERSET_INVALID_CHARACTER)
+          fprintf(stderr, "Invalid character");
+        else if (x_height_snapping_exceptions == NUMBERSET_OVERFLOW)
+          fprintf(stderr, "Overflow");
+        else if (x_height_snapping_exceptions == NUMBERSET_INVALID_RANGE)
+          fprintf(stderr, "Invalid range");
+        else if (x_height_snapping_exceptions == NUMBERSET_OVERLAPPING_RANGES)
+          fprintf(stderr, "Overlapping ranges");
+        else if (x_height_snapping_exceptions == NUMBERSET_NOT_ASCENDING)
+          fprintf(stderr, "Values und ranges must be ascending");
+        fprintf(stderr, " in x height snapping exceptions:\n"
+                        "  \"%s\"\n"
+                        "   %*s\n",
+                        x_height_snapping_exceptions_string,
+                        s - x_height_snapping_exceptions_string + 1, "^");
+      }
+      exit(EXIT_FAILURE);
+    }
+  }
+
   int num_args = argc - optind;
 
   if (num_args > 2)
@@ -598,9 +652,6 @@ main(int argc,
     in = stdin;
   }
 
-  unsigned char version_data[128];
-  unsigned char version_data_wide[256];
-
   const unsigned char* error_string;
   Progress_Data progress_data = {-1, 1, 0};
   Info_Data info_data;
@@ -609,8 +660,10 @@ main(int argc,
     info_func = NULL;
   else
   {
-    info_data.data = version_data;
-    info_data.data_wide = version_data_wide;
+    info_data.data = NULL; // must be deallocated after use
+    info_data.data_wide = NULL; // must be deallocated after use
+    info_data.data_len = 0;
+    info_data.data_wide_len = 0;
 
     info_data.hinting_range_min = hinting_range_min;
     info_data.hinting_range_max = hinting_range_max;
@@ -620,13 +673,21 @@ main(int argc,
     info_data.gdi_cleartype_strong_stem_width = gdi_cleartype_strong_stem_width;
     info_data.dw_cleartype_strong_stem_width = dw_cleartype_strong_stem_width;
 
+    info_data.windows_compatibility = windows_compatibility;
     info_data.pre_hinting = pre_hinting;
     info_data.hint_with_components = hint_with_components;
     info_data.increase_x_height = increase_x_height;
+    info_data.x_height_snapping_exceptions = x_height_snapping_exceptions;
     info_data.latin_fallback = latin_fallback;
     info_data.symbol = symbol;
 
-    build_version_string(&info_data);
+    int ret = build_version_string(&info_data);
+    if (ret == 1)
+      fprintf(stderr, "Warning: Can't allocate memory"
+                      " for ttfautohint options string in `name' table\n");
+    else if (ret == 2)
+      fprintf(stderr, "Warning: ttfautohint options string"
+                      " in `name' table too long\n");
   }
 
   if (in == stdin)
@@ -642,8 +703,10 @@ main(int argc,
                  "error-string,"
                  "progress-callback, progress-callback-data,"
                  "info-callback, info-callback-data,"
-                 "ignore-restrictions, pre-hinting, hint-with-components,"
-                 "increase-x-height, fallback-script, symbol,"
+                 "ignore-restrictions, windows-compatibility,"
+                 "pre-hinting, hint-with-components,"
+                 "increase-x-height, x-height-snapping-exceptions,"
+                 "fallback-script, symbol,"
                  "debug",
                  in, out,
                  hinting_range_min, hinting_range_max, hinting_limit,
@@ -652,10 +715,20 @@ main(int argc,
                  &error_string,
                  progress_func, &progress_data,
                  info_func, &info_data,
-                 ignore_restrictions, pre_hinting, hint_with_components,
-                 increase_x_height, latin_fallback, symbol,
+                 ignore_restrictions, windows_compatibility,
+                 pre_hinting, hint_with_components,
+                 increase_x_height, x_height_snapping_exceptions_string,
+                 latin_fallback, symbol,
                  debug);
 
+  if (!no_info)
+  {
+    free(info_data.data);
+    free(info_data.data_wide);
+  }
+
+  number_set_free(x_height_snapping_exceptions);
+
   if (error)
   {
     if (error == TA_Err_Invalid_FreeType_Version)
@@ -721,8 +794,9 @@ main(int argc,
 
   Main_GUI gui(hinting_range_min, hinting_range_max, hinting_limit,
                gray_strong_stem_width, gdi_cleartype_strong_stem_width,
-               dw_cleartype_strong_stem_width,
-               increase_x_height, ignore_restrictions, pre_hinting,
+               dw_cleartype_strong_stem_width, increase_x_height,
+               x_height_snapping_exceptions_string,
+               ignore_restrictions, windows_compatibility, pre_hinting,
                hint_with_components, no_info, latin_fallback, symbol);
   gui.show();
 
