@@ -20,12 +20,6 @@
 #undef MISSING
 #define MISSING (FT_UShort)~0
 
-/* a simple macro to emit bytecode instructions */
-#define BCI(code) *(bufp++) = (code)
-
-/* we increase the stack depth by this amount */
-#define ADDITIONAL_STACK_ELEMENTS 20
-
 
 #define DEBUGGING
 
@@ -91,6 +85,70 @@ FT_Byte ttfautohint_glyph_bytecode[7] =
 };
 
 
+/*
+ * convert array `args' into a sequence of NPUSHB, NPUSHW, PUSHB_X, and
+ * PUSHW_X instructions to be stored in `bufp' (the latter two instructions
+ * only if `optimize' is not set); if `need_words' is set, NPUSHW and
+ * PUSHW_X gets used
+ */
+
+FT_Byte*
+TA_build_push(FT_Byte* bufp,
+              FT_UInt* args,
+              FT_UInt num_args,
+              FT_Bool need_words,
+              FT_Bool optimize)
+{
+  FT_UInt* arg = args;
+  FT_UInt i, j, nargs;
+
+
+  if (need_words)
+  {
+    for (i = 0; i < num_args; i += 255)
+    {
+      nargs = (num_args - i > 255) ? 255 : num_args - i;
+
+      if (optimize && nargs <= 8)
+        BCI(PUSHW_1 - 1 + nargs);
+      else
+      {
+        BCI(NPUSHW);
+        BCI(nargs);
+      }
+      for (j = 0; j < nargs; j++)
+      {
+        BCI(HIGH(*arg));
+        BCI(LOW(*arg));
+        arg++;
+      }
+    }
+  }
+  else
+  {
+    for (i = 0; i < num_args; i += 255)
+    {
+      nargs = (num_args - i > 255) ? 255 : num_args - i;
+
+      if (optimize && nargs <= 8)
+        BCI(PUSHB_1 - 1 + nargs);
+      else
+      {
+        BCI(NPUSHB);
+        BCI(nargs);
+      }
+      for (j = 0; j < nargs; j++)
+      {
+        BCI(*arg);
+        arg++;
+      }
+    }
+  }
+
+  return bufp;
+}
+
+
 /* We add a subglyph for each composite glyph. */
 /* Since subglyphs must contain at least one point, */
 /* we have to adjust all point indices accordingly. */
@@ -142,13 +200,11 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
   FT_UInt* args;
   FT_UInt* arg;
   FT_UInt num_args;
-  FT_UInt nargs;
   FT_UShort num_segments;
 
   FT_Bool need_words = 0;
 
   FT_Int n;
-  FT_UInt i, j;
   FT_UInt base;
   FT_UShort num_packed_segments;
   FT_UShort num_storage;
@@ -296,54 +352,11 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
       *(arg--) = TA_adjust_point_index(recorder, last);
     }
   }
+
   /* with most fonts it is very rare */
   /* that any of the pushed arguments is larger than 0xFF, */
   /* thus we refrain from further optimizing this case */
-
-  arg = args;
-
-  if (need_words)
-  {
-    for (i = 0; i < num_args; i += 255)
-    {
-      nargs = (num_args - i > 255) ? 255 : num_args - i;
-
-
-      if (optimize && nargs <= 8)
-        BCI(PUSHW_1 - 1 + nargs);
-      else
-      {
-        BCI(NPUSHW);
-        BCI(nargs);
-      }
-      for (j = 0; j < nargs; j++)
-      {
-        BCI(HIGH(*arg));
-        BCI(LOW(*arg));
-        arg++;
-      }
-    }
-  }
-  else
-  {
-    for (i = 0; i < num_args; i += 255)
-    {
-      nargs = (num_args - i > 255) ? 255 : num_args - i;
-
-      if (optimize && nargs <= 8)
-        BCI(PUSHB_1 - 1 + nargs);
-      else
-      {
-        BCI(NPUSHB);
-        BCI(nargs);
-      }
-      for (j = 0; j < nargs; j++)
-      {
-        BCI(*arg);
-        arg++;
-      }
-    }
-  }
+  bufp = TA_build_push(bufp, args, num_args, need_words, optimize);
 
   BCI(CALL);
 
@@ -381,11 +394,9 @@ TA_sfnt_build_glyph_scaler(SFNT* sfnt,
   FT_UInt* args;
   FT_UInt* arg;
   FT_UInt num_args;
-  FT_UInt nargs;
 
   FT_Bool need_words = 0;
   FT_Int p, q;
-  FT_UInt i, j;
   FT_Int start, end;
   FT_UShort num_stack_elements;
 
@@ -449,50 +460,7 @@ TA_sfnt_build_glyph_scaler(SFNT* sfnt,
   /* with most fonts it is very rare */
   /* that any of the pushed arguments is larger than 0xFF, */
   /* thus we refrain from further optimizing this case */
-
-  arg = args;
-
-  if (need_words)
-  {
-    for (i = 0; i < num_args; i += 255)
-    {
-      nargs = (num_args - i > 255) ? 255 : num_args - i;
-
-      if (nargs <= 8)
-        BCI(PUSHW_1 - 1 + nargs);
-      else
-      {
-        BCI(NPUSHW);
-        BCI(nargs);
-      }
-      for (j = 0; j < nargs; j++)
-      {
-        BCI(HIGH(*arg));
-        BCI(LOW(*arg));
-        arg++;
-      }
-    }
-  }
-  else
-  {
-    for (i = 0; i < num_args; i += 255)
-    {
-      nargs = (num_args - i > 255) ? 255 : num_args - i;
-
-      if (nargs <= 8)
-        BCI(PUSHB_1 - 1 + nargs);
-      else
-      {
-        BCI(NPUSHB);
-        BCI(nargs);
-      }
-      for (j = 0; j < nargs; j++)
-      {
-        BCI(*arg);
-        arg++;
-      }
-    }
-  }
+  bufp = TA_build_push(bufp, args, num_args, need_words, 1);
 
   BCI(CALL);
 
@@ -1560,7 +1528,7 @@ TA_init_recorder(Recorder* recorder,
 
   recorder->wrap_around_segments =
     (FT_UShort*)malloc(recorder->num_wrap_around_segments
-                     * sizeof (FT_UShort));
+                       * sizeof (FT_UShort));
   if (!recorder->wrap_around_segments)
     return FT_Err_Out_Of_Memory;
 
@@ -1831,6 +1799,19 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
     if (error)
       goto Err;
 
+#ifdef DEBUGGING
+    if (font->debug)
+    {
+      int num_chars, i;
+
+
+      num_chars = fprintf(stderr, "size %d\n", size);
+      for (i = 0; i < num_chars - 1; i++)
+        putc('-', stderr);
+      fprintf(stderr, "\n\n");
+    }
+#endif
+
     /* calling `ta_loader_load_glyph' uses the */
     /* `TA_hints_recorder' function as a callback, */
     /* modifying `hints_record' */
@@ -1845,16 +1826,8 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
 #ifdef DEBUGGING
       if (font->debug)
       {
-        int num_chars, i;
-
-
         have_dumps = 1;
 
-        num_chars = fprintf(stderr, "size %d\n", size);
-        for (i = 0; i < num_chars - 1; i++)
-          putc('-', stderr);
-        fprintf(stderr, "\n\n");
-
         ta_glyph_hints_dump_edges(_ta_debug_hints);
         ta_glyph_hints_dump_segments(_ta_debug_hints);
         ta_glyph_hints_dump_points(_ta_debug_hints);
