@@ -292,6 +292,13 @@
 #define MIRP_rp0_keep_round_3           0xFF
 
 
+/* a simple macro to emit bytecode instructions */
+#define BCI(code) *(bufp++) = (code)
+
+/* we increase the stack depth by this amount */
+#define ADDITIONAL_STACK_ELEMENTS 20
+
+
 /* symbolic names for storage area locations */
 
 #define sal_i 0
@@ -324,12 +331,14 @@
 #define bci_decrement_component_counter bci_blue_round + 1
 #define bci_get_point_extrema bci_decrement_component_counter + 1
 #define bci_nibbles bci_get_point_extrema + 1
-
-/* 10 */
-#define bci_create_segment bci_nibbles + 1
-#define bci_create_segments bci_create_segment + 1
+#define bci_number_set_is_element bci_nibbles + 1
+#define bci_number_set_is_element2 bci_number_set_is_element + 1
 
 /* 12 */
+#define bci_create_segment bci_number_set_is_element2 + 1
+#define bci_create_segments bci_create_segment + 1
+
+/* 14 */
 /* the next ten entries must stay in this order */
 #define bci_create_segments_0 bci_create_segments + 1
 #define bci_create_segments_1 bci_create_segments_0 + 1
@@ -344,7 +353,7 @@
 
 #define bci_create_segments_composite bci_create_segments_9 + 1
 
-/* 23 */
+/* 25 */
 /* the next ten entries must stay in this order */
 #define bci_create_segments_composite_0 bci_create_segments_composite + 1
 #define bci_create_segments_composite_1 bci_create_segments_composite_0 + 1
@@ -357,24 +366,24 @@
 #define bci_create_segments_composite_8 bci_create_segments_composite_7 + 1
 #define bci_create_segments_composite_9 bci_create_segments_composite_8 + 1
 
-/* 33 */
+/* 35 */
 #define bci_align_segment bci_create_segments_composite_9 + 1
 #define bci_align_segments bci_align_segment + 1
 
-/* 35 */
+/* 37 */
 #define bci_scale_contour bci_align_segments + 1
 #define bci_scale_glyph bci_scale_contour + 1
 #define bci_scale_composite_glyph bci_scale_glyph + 1
 #define bci_shift_contour bci_scale_composite_glyph + 1
 #define bci_shift_subglyph bci_shift_contour + 1
 
-/* 40 */
+/* 42 */
 #define bci_ip_outer_align_point bci_shift_subglyph + 1
 #define bci_ip_on_align_points bci_ip_outer_align_point + 1
 #define bci_ip_between_align_point bci_ip_on_align_points + 1
 #define bci_ip_between_align_points bci_ip_between_align_point + 1
 
-/* 44 */
+/* 46 */
 #define bci_adjust_common bci_ip_between_align_points + 1
 #define bci_stem_common bci_adjust_common + 1
 #define bci_serif_common bci_stem_common + 1
@@ -382,12 +391,12 @@
 #define bci_serif_link1_common bci_serif_anchor_common + 1
 #define bci_serif_link2_common bci_serif_link1_common + 1
 
-/* 50 */
+/* 52 */
 #define bci_lower_bound bci_serif_link2_common + 1
 #define bci_upper_bound bci_lower_bound + 1
 #define bci_upper_lower_bound bci_upper_bound + 1
 
-/* 53 */
+/* 55 */
 #define bci_adjust_bound bci_upper_lower_bound + 1
 #define bci_stem_bound bci_adjust_bound + 1
 #define bci_link bci_stem_bound + 1
@@ -398,13 +407,13 @@
 /* the order of the `bci_action_*' entries must correspond */
 /* to the order of the TA_Action enumeration entries (in `tahints.h') */
 
-/* 59 */
+/* 61 */
 #define bci_action_ip_before bci_stem + 1
 #define bci_action_ip_after bci_action_ip_before + 1
 #define bci_action_ip_on bci_action_ip_after + 1
 #define bci_action_ip_between bci_action_ip_on + 1
 
-/* 63 */
+/* 65 */
 #define bci_action_blue bci_action_ip_between + 1
 #define bci_action_blue_anchor bci_action_blue + 1
 
@@ -414,7 +423,7 @@
 #define bci_action_anchor_round bci_action_anchor_serif + 1
 #define bci_action_anchor_round_serif bci_action_anchor_round + 1
 
-/* 69 */
+/* 71 */
 #define bci_action_adjust bci_action_anchor_round_serif + 1
 #define bci_action_adjust_serif bci_action_adjust + 1
 #define bci_action_adjust_round bci_action_adjust_serif + 1
@@ -424,13 +433,13 @@
 #define bci_action_adjust_bound_round bci_action_adjust_bound_serif + 1
 #define bci_action_adjust_bound_round_serif bci_action_adjust_bound_round + 1
 
-/* 77 */
+/* 79 */
 #define bci_action_link bci_action_adjust_bound_round_serif + 1
 #define bci_action_link_serif bci_action_link + 1
 #define bci_action_link_round bci_action_link_serif + 1
 #define bci_action_link_round_serif bci_action_link_round + 1
 
-/* 81 */
+/* 83 */
 #define bci_action_stem bci_action_link_round_serif + 1
 #define bci_action_stem_serif bci_action_stem + 1
 #define bci_action_stem_round bci_action_stem_serif + 1
@@ -440,31 +449,31 @@
 #define bci_action_stem_bound_round bci_action_stem_bound_serif + 1
 #define bci_action_stem_bound_round_serif bci_action_stem_bound_round + 1
 
-/* 89 */
+/* 91 */
 #define bci_action_serif bci_action_stem_bound_round_serif + 1
 #define bci_action_serif_lower_bound bci_action_serif + 1
 #define bci_action_serif_upper_bound bci_action_serif_lower_bound + 1
 #define bci_action_serif_upper_lower_bound bci_action_serif_upper_bound + 1
 
-/* 93 */
+/* 95 */
 #define bci_action_serif_anchor bci_action_serif_upper_lower_bound + 1
 #define bci_action_serif_anchor_lower_bound bci_action_serif_anchor + 1
 #define bci_action_serif_anchor_upper_bound bci_action_serif_anchor_lower_bound + 1
 #define bci_action_serif_anchor_upper_lower_bound bci_action_serif_anchor_upper_bound + 1
 
-/* 97 */
+/* 99 */
 #define bci_action_serif_link1 bci_action_serif_anchor_upper_lower_bound + 1
 #define bci_action_serif_link1_lower_bound bci_action_serif_link1 + 1
 #define bci_action_serif_link1_upper_bound bci_action_serif_link1_lower_bound + 1
 #define bci_action_serif_link1_upper_lower_bound bci_action_serif_link1_upper_bound + 1
 
-/* 101 */
+/* 103 */
 #define bci_action_serif_link2 bci_action_serif_link1_upper_lower_bound + 1
 #define bci_action_serif_link2_lower_bound bci_action_serif_link2 + 1
 #define bci_action_serif_link2_upper_bound bci_action_serif_link2_lower_bound + 1
 #define bci_action_serif_link2_upper_lower_bound bci_action_serif_link2_upper_bound + 1
 
-/* 105 */
+/* 107 */
 #define bci_hint_glyph bci_action_serif_link2_upper_lower_bound + 1
 
 #define NUM_FDEFS bci_hint_glyph + 1 /* must be last */
@@ -483,7 +492,9 @@
 #define cvtl_is_extra_light cvtl_funits_to_pixels + 1
 #define cvtl_is_subglyph cvtl_is_extra_light + 1
 #define cvtl_stem_width_function cvtl_is_subglyph + 1
-#define cvtl_max_runtime cvtl_stem_width_function + 1 /* must be last */
+#define cvtl_is_element cvtl_stem_width_function + 1
+
+#define cvtl_max_runtime cvtl_is_element + 1 /* must be last */
 
 /* symbolic names for compile-time CVT locations */
 /* (assigned in `cvt') */
@@ -509,11 +520,11 @@
            ? 0 \
            : ((TA_LatinMetrics)font->loader->hints.metrics)->axis[1].width_count)
 
-/* the number of blue zones */
+/* the number of blue zones (including the artificial ones) */
 #define CVT_BLUES_SIZE(font) \
           ((font->loader->hints.metrics->clazz->script == TA_SCRIPT_DUMMY) \
            ? 0 \
-           : ((TA_LatinMetrics)font->loader->hints.metrics)->axis[1].blue_count)
+           : ((TA_LatinMetrics)font->loader->hints.metrics)->axis[1].blue_count + 2)
 
 /* the blue zone values for flat and round edges */
 #define CVT_BLUE_REFS_OFFSET(font) \
