@@ -866,6 +866,100 @@ unsigned char FPGM(bci_nibbles) [] =
 };
 
 
+/*
+ * bci_number_set_is_element
+ *
+ *   Pop values from stack until it is empty.  If one of them is equal to
+ *   the current PPEM value, set `cvtl_is_element' to 1 (and to 0
+ *   otherwise).
+ *
+ * in: ppem_value_1
+ *     ppem_value_2
+ *     ...
+ *
+ * CVT: cvtl_is_element
+ */
+
+unsigned char FPGM(bci_number_set_is_element) [] =
+{
+
+  PUSHB_1,
+    bci_number_set_is_element,
+  FDEF,
+
+/* start_loop: */
+  MPPEM,
+  EQ,
+  IF,
+    PUSHB_2,
+      cvtl_is_element,
+      1,
+    WCVTP,
+  EIF,
+
+  DEPTH,
+  PUSHB_1,
+    13,
+  NEG,
+  SWAP,
+  JROT, /* goto start_loop if stack depth != 0 */
+
+  ENDF,
+
+};
+
+
+/*
+ * bci_number_set_is_element2
+ *
+ *   Pop value ranges from stack until it is empty.  If one of them contains
+ *   the current PPEM value, set `cvtl_is_element' to 1 (and to 0
+ *   otherwise).
+ *
+ * in: ppem_range_1_start
+ *     ppem_range_1_end
+ *     ppem_range_2_start
+ *     ppem_range_2_end
+ *     ...
+ *
+ * CVT: cvtl_is_element
+ */
+
+unsigned char FPGM(bci_number_set_is_element2) [] =
+{
+
+  PUSHB_1,
+    bci_number_set_is_element2,
+  FDEF,
+
+/* start_loop: */
+  MPPEM,
+  LTEQ,
+  IF,
+    MPPEM,
+    GTEQ,
+    IF,
+      PUSHB_2,
+        cvtl_is_element,
+        1,
+      WCVTP,
+    EIF,
+  ELSE,
+    POP,
+  EIF,
+
+  DEPTH,
+  PUSHB_1,
+    19,
+  NEG,
+  SWAP,
+  JROT, /* goto start_loop if stack depth != 0 */
+
+  ENDF,
+
+};
+
+
 /*
  * bci_create_segment
  *
@@ -2152,11 +2246,18 @@ unsigned char FPGM(bci_ip_between_align_points) [] =
   PUSHB_1,
     2,
   CINDEX, /* s: ... before after before after */
-  MD_cur, /* b = after_pos - before_pos */
+  MD_cur, /* a = after_pos - before_pos */
   ROLL,
   ROLL,
-  MD_orig_ZP2_0, /* a = after_orig_pos - before_orig_pos */
-  DIV, /* s: a/b */
+  MD_orig_ZP2_0, /* b = after_orig_pos - before_orig_pos */
+
+  DUP,
+  IF, /* b != 0 ? */
+    DIV, /* s: a/b */
+  ELSE,
+    POP, /* avoid division by zero */
+  EIF,
+
   PUSHB_1,
     sal_j,
   SWAP,
@@ -4355,7 +4456,13 @@ unsigned char FPGM(bci_serif_link1_common) [] =
     RCVT,
     MUL, /* (b-c) in 16.16 format */
     SWAP,
-    DIV, /* s: [...] after edge before a a (b-c)/c */
+
+    DUP,
+    IF, /* c != 0 ? */
+      DIV, /* s: [...] after edge before a a (b-c)/c */
+    ELSE,
+      POP, /* avoid division by zero */
+    EIF,
 
     MUL, /* a * (b-c)/c * 2^10 */
     PUSHB_1,
@@ -4839,6 +4946,12 @@ TA_table_build_fpgm(FT_Byte** fpgm,
   FT_Byte* buf_p;
 
 
+  /* for compatibility with dumb bytecode interpreters or analyzers, */
+  /* FDEFs are stored in ascending index order, without holes -- */
+  /* note that some FDEFs are not always needed */
+  /* (depending on options of `TTFautohint'), */
+  /* but implementing dynamic FDEF indices would be a lot of work */
+
   buf_len = sizeof (FPGM(bci_round))
             + sizeof (FPGM(bci_smooth_stem_width_a))
             + 1
@@ -4857,9 +4970,12 @@ TA_table_build_fpgm(FT_Byte** fpgm,
             + sizeof (FPGM(bci_decrement_component_counter))
             + sizeof (FPGM(bci_get_point_extrema))
             + sizeof (FPGM(bci_nibbles))
+            + sizeof (FPGM(bci_number_set_is_element))
+            + sizeof (FPGM(bci_number_set_is_element2))
 
             + sizeof (FPGM(bci_create_segment))
             + sizeof (FPGM(bci_create_segments))
+
             + sizeof (FPGM(bci_create_segments_0))
             + sizeof (FPGM(bci_create_segments_1))
             + sizeof (FPGM(bci_create_segments_2))
@@ -4870,7 +4986,9 @@ TA_table_build_fpgm(FT_Byte** fpgm,
             + sizeof (FPGM(bci_create_segments_7))
             + sizeof (FPGM(bci_create_segments_8))
             + sizeof (FPGM(bci_create_segments_9))
+
             + sizeof (FPGM(bci_create_segments_composite))
+
             + sizeof (FPGM(bci_create_segments_composite_0))
             + sizeof (FPGM(bci_create_segments_composite_1))
             + sizeof (FPGM(bci_create_segments_composite_2))
@@ -4881,6 +4999,7 @@ TA_table_build_fpgm(FT_Byte** fpgm,
             + sizeof (FPGM(bci_create_segments_composite_7))
             + sizeof (FPGM(bci_create_segments_composite_8))
             + sizeof (FPGM(bci_create_segments_composite_9))
+
             + sizeof (FPGM(bci_align_segment))
             + sizeof (FPGM(bci_align_segments))
 
@@ -4906,55 +5025,64 @@ TA_table_build_fpgm(FT_Byte** fpgm,
             + sizeof (FPGM(bci_upper_bound))
             + sizeof (FPGM(bci_upper_lower_bound))
 
+            + sizeof (FPGM(bci_adjust_bound))
+            + sizeof (FPGM(bci_stem_bound))
+            + sizeof (FPGM(bci_link))
+            + sizeof (FPGM(bci_anchor))
+            + sizeof (FPGM(bci_adjust))
+            + sizeof (FPGM(bci_stem))
+
             + sizeof (FPGM(bci_action_ip_before))
             + sizeof (FPGM(bci_action_ip_after))
             + sizeof (FPGM(bci_action_ip_on))
             + sizeof (FPGM(bci_action_ip_between))
 
-            + sizeof (FPGM(bci_adjust_bound))
-            + sizeof (FPGM(bci_action_adjust_bound))
-            + sizeof (FPGM(bci_action_adjust_bound_serif))
-            + sizeof (FPGM(bci_action_adjust_bound_round))
-            + sizeof (FPGM(bci_action_adjust_bound_round_serif))
-            + sizeof (FPGM(bci_stem_bound))
-            + sizeof (FPGM(bci_action_stem_bound))
-            + sizeof (FPGM(bci_action_stem_bound_serif))
-            + sizeof (FPGM(bci_action_stem_bound_round))
-            + sizeof (FPGM(bci_action_stem_bound_round_serif))
-            + sizeof (FPGM(bci_link))
-            + sizeof (FPGM(bci_action_link))
-            + sizeof (FPGM(bci_action_link_serif))
-            + sizeof (FPGM(bci_action_link_round))
-            + sizeof (FPGM(bci_action_link_round_serif))
-            + sizeof (FPGM(bci_anchor))
+            + sizeof (FPGM(bci_action_blue))
+            + sizeof (FPGM(bci_action_blue_anchor))
+
             + sizeof (FPGM(bci_action_anchor))
             + sizeof (FPGM(bci_action_anchor_serif))
             + sizeof (FPGM(bci_action_anchor_round))
             + sizeof (FPGM(bci_action_anchor_round_serif))
-            + sizeof (FPGM(bci_action_blue_anchor))
-            + sizeof (FPGM(bci_adjust))
+
             + sizeof (FPGM(bci_action_adjust))
             + sizeof (FPGM(bci_action_adjust_serif))
             + sizeof (FPGM(bci_action_adjust_round))
             + sizeof (FPGM(bci_action_adjust_round_serif))
-            + sizeof (FPGM(bci_stem))
+            + sizeof (FPGM(bci_action_adjust_bound))
+            + sizeof (FPGM(bci_action_adjust_bound_serif))
+            + sizeof (FPGM(bci_action_adjust_bound_round))
+            + sizeof (FPGM(bci_action_adjust_bound_round_serif))
+
+            + sizeof (FPGM(bci_action_link))
+            + sizeof (FPGM(bci_action_link_serif))
+            + sizeof (FPGM(bci_action_link_round))
+            + sizeof (FPGM(bci_action_link_round_serif))
+
             + sizeof (FPGM(bci_action_stem))
             + sizeof (FPGM(bci_action_stem_serif))
             + sizeof (FPGM(bci_action_stem_round))
             + sizeof (FPGM(bci_action_stem_round_serif))
-            + sizeof (FPGM(bci_action_blue))
+            + sizeof (FPGM(bci_action_stem_bound))
+            + sizeof (FPGM(bci_action_stem_bound_serif))
+            + sizeof (FPGM(bci_action_stem_bound_round))
+            + sizeof (FPGM(bci_action_stem_bound_round_serif))
+
             + sizeof (FPGM(bci_action_serif))
             + sizeof (FPGM(bci_action_serif_lower_bound))
             + sizeof (FPGM(bci_action_serif_upper_bound))
             + sizeof (FPGM(bci_action_serif_upper_lower_bound))
+
             + sizeof (FPGM(bci_action_serif_anchor))
             + sizeof (FPGM(bci_action_serif_anchor_lower_bound))
             + sizeof (FPGM(bci_action_serif_anchor_upper_bound))
             + sizeof (FPGM(bci_action_serif_anchor_upper_lower_bound))
+
             + sizeof (FPGM(bci_action_serif_link1))
             + sizeof (FPGM(bci_action_serif_link1_lower_bound))
             + sizeof (FPGM(bci_action_serif_link1_upper_bound))
             + sizeof (FPGM(bci_action_serif_link1_upper_lower_bound))
+
             + sizeof (FPGM(bci_action_serif_link2))
             + sizeof (FPGM(bci_action_serif_link2_lower_bound))
             + sizeof (FPGM(bci_action_serif_link2_upper_bound))
@@ -4995,9 +5123,12 @@ TA_table_build_fpgm(FT_Byte** fpgm,
   COPY_FPGM(bci_decrement_component_counter);
   COPY_FPGM(bci_get_point_extrema);
   COPY_FPGM(bci_nibbles);
+  COPY_FPGM(bci_number_set_is_element);
+  COPY_FPGM(bci_number_set_is_element2);
 
   COPY_FPGM(bci_create_segment);
   COPY_FPGM(bci_create_segments);
+
   COPY_FPGM(bci_create_segments_0);
   COPY_FPGM(bci_create_segments_1);
   COPY_FPGM(bci_create_segments_2);
@@ -5008,7 +5139,9 @@ TA_table_build_fpgm(FT_Byte** fpgm,
   COPY_FPGM(bci_create_segments_7);
   COPY_FPGM(bci_create_segments_8);
   COPY_FPGM(bci_create_segments_9);
+
   COPY_FPGM(bci_create_segments_composite);
+
   COPY_FPGM(bci_create_segments_composite_0);
   COPY_FPGM(bci_create_segments_composite_1);
   COPY_FPGM(bci_create_segments_composite_2);
@@ -5019,6 +5152,7 @@ TA_table_build_fpgm(FT_Byte** fpgm,
   COPY_FPGM(bci_create_segments_composite_7);
   COPY_FPGM(bci_create_segments_composite_8);
   COPY_FPGM(bci_create_segments_composite_9);
+
   COPY_FPGM(bci_align_segment);
   COPY_FPGM(bci_align_segments);
 
@@ -5044,55 +5178,64 @@ TA_table_build_fpgm(FT_Byte** fpgm,
   COPY_FPGM(bci_upper_bound);
   COPY_FPGM(bci_upper_lower_bound);
 
+  COPY_FPGM(bci_adjust_bound);
+  COPY_FPGM(bci_stem_bound);
+  COPY_FPGM(bci_link);
+  COPY_FPGM(bci_anchor);
+  COPY_FPGM(bci_adjust);
+  COPY_FPGM(bci_stem);
+
   COPY_FPGM(bci_action_ip_before);
   COPY_FPGM(bci_action_ip_after);
   COPY_FPGM(bci_action_ip_on);
   COPY_FPGM(bci_action_ip_between);
 
-  COPY_FPGM(bci_adjust_bound);
-  COPY_FPGM(bci_action_adjust_bound);
-  COPY_FPGM(bci_action_adjust_bound_serif);
-  COPY_FPGM(bci_action_adjust_bound_round);
-  COPY_FPGM(bci_action_adjust_bound_round_serif);
-  COPY_FPGM(bci_stem_bound);
-  COPY_FPGM(bci_action_stem_bound);
-  COPY_FPGM(bci_action_stem_bound_serif);
-  COPY_FPGM(bci_action_stem_bound_round);
-  COPY_FPGM(bci_action_stem_bound_round_serif);
-  COPY_FPGM(bci_link);
-  COPY_FPGM(bci_action_link);
-  COPY_FPGM(bci_action_link_serif);
-  COPY_FPGM(bci_action_link_round);
-  COPY_FPGM(bci_action_link_round_serif);
-  COPY_FPGM(bci_anchor);
+  COPY_FPGM(bci_action_blue);
+  COPY_FPGM(bci_action_blue_anchor);
+
   COPY_FPGM(bci_action_anchor);
   COPY_FPGM(bci_action_anchor_serif);
   COPY_FPGM(bci_action_anchor_round);
   COPY_FPGM(bci_action_anchor_round_serif);
-  COPY_FPGM(bci_action_blue_anchor);
-  COPY_FPGM(bci_adjust);
+
   COPY_FPGM(bci_action_adjust);
   COPY_FPGM(bci_action_adjust_serif);
   COPY_FPGM(bci_action_adjust_round);
   COPY_FPGM(bci_action_adjust_round_serif);
-  COPY_FPGM(bci_stem);
+  COPY_FPGM(bci_action_adjust_bound);
+  COPY_FPGM(bci_action_adjust_bound_serif);
+  COPY_FPGM(bci_action_adjust_bound_round);
+  COPY_FPGM(bci_action_adjust_bound_round_serif);
+
+  COPY_FPGM(bci_action_link);
+  COPY_FPGM(bci_action_link_serif);
+  COPY_FPGM(bci_action_link_round);
+  COPY_FPGM(bci_action_link_round_serif);
+
   COPY_FPGM(bci_action_stem);
   COPY_FPGM(bci_action_stem_serif);
   COPY_FPGM(bci_action_stem_round);
   COPY_FPGM(bci_action_stem_round_serif);
-  COPY_FPGM(bci_action_blue);
+  COPY_FPGM(bci_action_stem_bound);
+  COPY_FPGM(bci_action_stem_bound_serif);
+  COPY_FPGM(bci_action_stem_bound_round);
+  COPY_FPGM(bci_action_stem_bound_round_serif);
+
   COPY_FPGM(bci_action_serif);
   COPY_FPGM(bci_action_serif_lower_bound);
   COPY_FPGM(bci_action_serif_upper_bound);
   COPY_FPGM(bci_action_serif_upper_lower_bound);
+
   COPY_FPGM(bci_action_serif_anchor);
   COPY_FPGM(bci_action_serif_anchor_lower_bound);
   COPY_FPGM(bci_action_serif_anchor_upper_bound);
   COPY_FPGM(bci_action_serif_anchor_upper_lower_bound);
+
   COPY_FPGM(bci_action_serif_link1);
   COPY_FPGM(bci_action_serif_link1_lower_bound);
   COPY_FPGM(bci_action_serif_link1_upper_bound);
   COPY_FPGM(bci_action_serif_link1_upper_lower_bound);
+
   COPY_FPGM(bci_action_serif_link2);
   COPY_FPGM(bci_action_serif_link2_lower_bound);
   COPY_FPGM(bci_action_serif_link2_upper_bound);
