@@ -16,7 +16,7 @@
 #include "ta.h"
 
 
-FT_Error
+static FT_Error
 TA_sfnt_build_glyf_hints(SFNT* sfnt,
                          FONT* font)
 {
@@ -116,7 +116,8 @@ static FT_Error
 TA_glyph_parse_composite(GLYPH* glyph,
                          FT_Byte* buf,
                          FT_ULong len,
-                         FT_UShort num_glyphs)
+                         FT_UShort num_glyphs,
+                         FT_Bool hint_with_components)
 {
   FT_ULong flags_offset; /* after the loop, this is the offset */
                          /* to the last element in the flags array */
@@ -147,7 +148,7 @@ TA_glyph_parse_composite(GLYPH* glyph,
   /* which eventually becomes the last glyph in the `glyf' table; */
   /* for convenience, however, it is not added to the `components' array */
   /* (doing so simplifies the conversion of point indices later on) */
-  if (glyph->num_composite_contours)
+  if (glyph->num_composite_contours && hint_with_components)
   {
     FT_Short x_min;
     FT_Short x_max;
@@ -345,6 +346,18 @@ TA_glyph_parse_simple(GLYPH* glyph,
   num_ins = *(p++) << 8;
   num_ins += *(p++);
 
+  /* assure that we don't process a font */
+  /* which already contains a `.ttfautohint' glyph */
+  /* (a font with a `post' table version 3.0 doesn't contain glyph names, */
+  /* so we have to check it this way) */
+  if (glyph->num_points == 1
+      && num_ins >= sizeof (ttfautohint_glyph_bytecode))
+  {
+    if (!strncmp((char*)p, (char*)ttfautohint_glyph_bytecode,
+                 sizeof (ttfautohint_glyph_bytecode)))
+      return TA_Err_Already_Processed;
+  }
+
   p += num_ins;
 
   if (p > endp)
@@ -521,17 +534,20 @@ TA_sfnt_compute_composite_pointsums(SFNT* sfnt,
 
       glyph->num_composite_contours = num_composite_contours;
 
-      /* update maximum values, */
-      /* including the subglyphs not in `components' array */
-      /* (each of them has a single point in a single contour) */
-      if (num_composite_points + glyph->num_pointsums
-          > sfnt->max_composite_points)
-        sfnt->max_composite_points = num_composite_points
-                                     + glyph->num_pointsums;
-      if (num_composite_contours + glyph->num_pointsums
-          > sfnt->max_composite_contours)
-        sfnt->max_composite_contours = num_composite_contours
+      if (font->hint_with_components)
+      {
+        /* update maximum values, */
+        /* including the subglyphs not in `components' array */
+        /* (each of them has a single point in a single contour) */
+        if (num_composite_points + glyph->num_pointsums
+            > sfnt->max_composite_points)
+          sfnt->max_composite_points = num_composite_points
                                        + glyph->num_pointsums;
+        if (num_composite_contours + glyph->num_pointsums
+            > sfnt->max_composite_contours)
+          sfnt->max_composite_contours = num_composite_contours
+                                         + glyph->num_pointsums;
+      }
     }
   }
 
@@ -580,12 +596,17 @@ TA_sfnt_split_glyf_table(SFNT* sfnt,
   loop_count = data->num_glyphs - 1;
 
   /* allocate one more glyph slot if we have composite glyphs */
-  if (!sfnt->max_components)
+  if (!sfnt->max_components || !font->hint_with_components)
     data->num_glyphs -= 1;
   data->glyphs = (GLYPH*)calloc(1, data->num_glyphs * sizeof (GLYPH));
   if (!data->glyphs)
     return FT_Err_Out_Of_Memory;
 
+  data->master_globals = NULL;
+  data->cvt_idx = MISSING;
+  data->fpgm_idx = MISSING;
+  data->prep_idx = MISSING;
+
   /* first loop over `loca' and `glyf' data */
 
   p = loca_table->buf;
@@ -669,7 +690,7 @@ TA_sfnt_split_glyf_table(SFNT* sfnt,
     }
   }
 
-  if (sfnt->max_components)
+  if (sfnt->max_components && font->hint_with_components)
   {
     error = TA_sfnt_compute_composite_pointsums(sfnt, font);
     if (error)
@@ -734,7 +755,9 @@ TA_sfnt_split_glyf_table(SFNT* sfnt,
       /* is more or less invalid. */
 
       if (glyph->num_contours < 0)
-        error = TA_glyph_parse_composite(glyph, buf, len, data->num_glyphs);
+        error = TA_glyph_parse_composite(glyph, buf, len,
+                                         data->num_glyphs,
+                                         font->hint_with_components);
       else
         error = TA_glyph_parse_simple(glyph, buf, len);
       if (error)
@@ -742,26 +765,12 @@ TA_sfnt_split_glyf_table(SFNT* sfnt,
     }
   }
 
-  if (sfnt->max_components)
+  if (sfnt->max_components && font->hint_with_components)
   {
     /* construct and append our special glyph used as a composite element */
     GLYPH* glyph = &data->glyphs[data->num_glyphs - 1];
     FT_Byte* buf;
 
-    FT_Byte bytecode[] =
-    {
-
-      /* increment `cvtl_is_subglyph' counter */
-      PUSHB_3,
-        cvtl_is_subglyph,
-        1,
-        cvtl_is_subglyph,
-      RCVT,
-      ADD,
-      WCVTP,
-
-    };
-
     glyph->len1 = 12;
     glyph->len2 = 1;
     glyph->buf = (FT_Byte*)malloc(glyph->len1 + glyph->len2);
@@ -788,11 +797,11 @@ TA_sfnt_split_glyf_table(SFNT* sfnt,
     /* add bytecode also; */
     /* this works because the loop in `TA_sfnt_build_glyf_hints' */
     /* doesn't include the newly appended glyph */
-    glyph->ins_len = sizeof (bytecode);
+    glyph->ins_len = sizeof (ttfautohint_glyph_bytecode);
     glyph->ins_buf = (FT_Byte*)malloc(glyph->ins_len);
     if (!glyph->ins_buf)
       return FT_Err_Out_Of_Memory;
-    memcpy(glyph->ins_buf, bytecode, glyph->ins_len);
+    memcpy(glyph->ins_buf, ttfautohint_glyph_bytecode, glyph->ins_len);
 
     sfnt->max_components += 1;
   }
@@ -805,6 +814,8 @@ FT_Error
 TA_sfnt_build_glyf_table(SFNT* sfnt,
                          FONT* font)
 {
+  FT_Error error;
+
   SFNT_Table* glyf_table = &font->tables[sfnt->glyf_idx];
   glyf_Data* data = (glyf_Data*)glyf_table->data;
 
@@ -819,6 +830,10 @@ TA_sfnt_build_glyf_table(SFNT* sfnt,
   if (glyf_table->processed)
     return TA_Err_Ok;
 
+  error = TA_sfnt_build_glyf_hints(sfnt, font);
+  if (error)
+    return error;
+
   /* get table size */
   len = 0;
   glyph = data->glyphs;
@@ -1188,4 +1203,136 @@ TA_sfnt_create_glyf_data(SFNT* sfnt,
   return TA_Err_Ok;
 }
 
+
+FT_Error
+TA_sfnt_handle_coverage(SFNT* sfnt,
+                        FONT* font)
+{
+  FT_Error error;
+
+  SFNT_Table* glyf_table = &font->tables[sfnt->glyf_idx];
+  glyf_Data* data = (glyf_Data*)glyf_table->data;
+
+  FT_Face face = sfnt->face;
+  TA_FaceGlobals curr_globals;
+
+  FT_UInt saved_fallback_script = font->fallback_script;
+
+
+  /* using TA_SCRIPT_NONE as the fallback script ensures */
+  /* that uncovered glyphs stay as-is */
+  /* (we handle the fallback script later on) */
+  font->fallback_script = TA_SCRIPT_NONE;
+
+  /* trigger computation of coverage */
+  error = ta_loader_init(font);
+  if (error)
+    goto Exit;
+  error = ta_loader_reset(font, face);
+  if (error)
+    goto Exit;
+  ta_loader_done(font);
+
+  font->fallback_script = saved_fallback_script;
+  curr_globals = (TA_FaceGlobals)face->autohint.data;
+
+  if (!data->master_globals)
+  {
+    /* initialize */
+    data->master_globals = curr_globals;
+    goto Exit;
+  }
+
+  /* we have the same `glyf' table for another subfont; */
+  /* merge the current coverage info into the `master' coverage info */
+  {
+    TA_FaceGlobals master_globals = data->master_globals;
+    FT_Long count = master_globals->glyph_count;
+
+    FT_Byte* master = master_globals->glyph_scripts;
+    FT_Byte* curr = curr_globals->glyph_scripts;
+
+    FT_Byte* limit = master + count;
+
+
+    /* we simply copy the data, */
+    /* assuming that a given glyph always has the same properties -- */
+    /* as soon as we make the script selection more fine-grained, */
+    /* it is possible that this assumption doesn't hold: */
+    /* for example, glyph `A' can be used for both Cyrillic and Latin */
+    while (master < limit)
+    {
+      if ((*curr & ~TA_DIGIT) != TA_SCRIPT_NONE)
+        *master = *curr;
+
+      master++;
+      curr++;
+    }
+  }
+
+Exit:
+  return error;
+}
+
+
+void
+TA_sfnt_adjust_master_coverage(SFNT* sfnt,
+                               FONT* font)
+{
+  SFNT_Table* glyf_table = &font->tables[sfnt->glyf_idx];
+  glyf_Data* data = (glyf_Data*)glyf_table->data;
+
+  FT_Face face = sfnt->face;
+
+  TA_FaceGlobals master_globals = data->master_globals;
+  TA_FaceGlobals curr_globals = (TA_FaceGlobals)face->autohint.data;
+
+
+  /* use fallback script for uncovered glyphs */
+  if (master_globals == curr_globals)
+  {
+    FT_Long nn;
+    FT_Byte* gscripts = master_globals->glyph_scripts;
+
+
+    for (nn = 0; nn < master_globals->glyph_count; nn++)
+    {
+      if ((gscripts[nn] & ~TA_DIGIT) == TA_SCRIPT_NONE)
+      {
+        gscripts[nn] &= ~TA_SCRIPT_NONE;
+        gscripts[nn] |= master_globals->font->fallback_script;
+      }
+    }
+  }
+}
+
+
+#if 0
+
+void
+TA_sfnt_copy_master_coverage(SFNT* sfnt,
+                             FONT* font)
+{
+  SFNT_Table* glyf_table = &font->tables[sfnt->glyf_idx];
+  glyf_Data* data = (glyf_Data*)glyf_table->data;
+
+  FT_Face face = sfnt->face;
+
+  TA_FaceGlobals master_globals = data->master_globals;
+  TA_FaceGlobals curr_globals = (TA_FaceGlobals)face->autohint.data;
+
+
+  if (master_globals != curr_globals)
+  {
+    FT_Long count = master_globals->glyph_count;
+    FT_Byte* master = master_globals->glyph_scripts;
+    FT_Byte* curr = curr_globals->glyph_scripts;
+
+
+    memcpy(curr, master, count);
+  }
+}
+
+#endif /* 0 */
+
 /* end of taglyf.c */
