@@ -21,6 +21,7 @@
 
 #include <ft2build.h>
 #include FT_ADVANCES_H
+#include FT_TRUETYPE_TABLES_H
 
 #include "taglobal.h"
 #include "talatin.h"
@@ -31,19 +32,23 @@
 #include "tawarp.h"
 #endif
 
+#include <numberset.h>
+
 
 /* find segments and links, compute all stem widths, and initialize */
 /* standard width and height for the glyph with given charcode */
 
 void
 ta_latin_metrics_init_widths(TA_LatinMetrics metrics,
-                             FT_Face face,
-                             FT_ULong charcode)
+                             FT_Face face)
 {
   /* scan the array of segments in each direction */
   TA_GlyphHintsRec hints[1];
 
 
+  TA_LOG(("standard widths computation\n"
+          "===========================\n\n"));
+
   ta_glyph_hints_init(hints);
 
   metrics->axis[TA_DIMENSION_HORZ].width_count = 0;
@@ -57,10 +62,14 @@ ta_latin_metrics_init_widths(TA_LatinMetrics metrics,
     TA_Scaler scaler = &dummy->root.scaler;
 
 
-    glyph_index = FT_Get_Char_Index(face, charcode);
+    glyph_index = FT_Get_Char_Index(face,
+                                    metrics->root.clazz->standard_char);
     if (glyph_index == 0)
       goto Exit;
 
+    TA_LOG(("standard character: 0x%X (glyph index %d)\n",
+            metrics->root.clazz->standard_char, glyph_index));
+
     error = FT_Load_Glyph(face, glyph_index, FT_LOAD_NO_SCALE);
     if (error || face->glyph->outline.n_points <= 0)
       goto Exit;
@@ -144,9 +153,28 @@ Exit:
       axis->edge_distance_threshold = stdw / 5;
       axis->standard_width = stdw;
       axis->extra_light = 0;
+
+#ifdef TA_DEBUG
+      {
+        FT_UInt i;
+
+
+        TA_LOG(("%s widths:\n",
+                dim == TA_DIMENSION_VERT ? "horizontal"
+                                         : "vertical"));
+
+        TA_LOG(("  %d (standard)", axis->standard_width));
+        for (i = 1; i < axis->width_count; i++)
+          TA_LOG((" %d", axis->widths[i].org));
+
+        TA_LOG(("\n"));
+      }
+#endif
     }
   }
 
+  TA_LOG(("\n"));
+
   ta_glyph_hints_done(hints);
 }
 
@@ -445,6 +473,51 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
             *blue_ref, *blue_shoot));
   }
 
+  /* add two blue zones for usWinAscent and usWinDescent */
+  /* just in case the above algorithm has missed them -- */
+  /* Windows cuts off everything outside of those two values */
+  {
+    TT_OS2* os2;
+
+
+    os2 = (TT_OS2*)FT_Get_Sfnt_Table(face, ft_sfnt_os2);
+
+    if (os2)
+    {
+      blue = &axis->blues[axis->blue_count];
+      blue->flags = TA_LATIN_BLUE_TOP | TA_LATIN_BLUE_ACTIVE;
+      blue->ref.org =
+      blue->shoot.org = os2->usWinAscent;
+
+      TA_LOG(("artificial blue zone for usWinAscent:\n"
+              "    -> reference = %ld\n"
+              "       overshoot = %ld\n",
+              blue->ref.org, blue->shoot.org));
+
+      blue = &axis->blues[axis->blue_count + 1];
+      blue->flags = TA_LATIN_BLUE_ACTIVE;
+      blue->ref.org =
+      blue->shoot.org = -os2->usWinDescent;
+
+      TA_LOG(("artificial blue zone for usWinDescent:\n"
+              "    -> reference = %ld\n"
+              "       overshoot = %ld\n",
+              blue->ref.org, blue->shoot.org));
+    }
+    else
+    {
+      blue = &axis->blues[axis->blue_count];
+      blue->flags =
+      blue->ref.org =
+      blue->shoot.org = 0;
+
+      blue = &axis->blues[axis->blue_count + 1];
+      blue->flags =
+      blue->ref.org =
+      blue->shoot.org = 0;
+    }
+  }
+
   TA_LOG(("\n"));
 
   return;
@@ -504,35 +577,14 @@ FT_Error
 ta_latin_metrics_init(TA_LatinMetrics metrics,
                       FT_Face face)
 {
-  FT_Error error = FT_Err_Ok;
   FT_CharMap oldmap = face->charmap;
-  FT_UInt ee;
-
-  static const FT_Encoding latin_encodings[] =
-  {
-    FT_ENCODING_UNICODE,
-    FT_ENCODING_APPLE_ROMAN,
-    FT_ENCODING_ADOBE_STANDARD,
-    FT_ENCODING_ADOBE_LATIN_1,
-
-    FT_ENCODING_NONE /* end of list */
-  };
 
 
   metrics->units_per_em = face->units_per_EM;
 
-  /* do we have a latin charmap in there? */
-  for (ee = 0; latin_encodings[ee] != FT_ENCODING_NONE; ee++)
+  if (!FT_Select_Charmap(face, FT_ENCODING_UNICODE))
   {
-    error = FT_Select_Charmap(face, latin_encodings[ee]);
-    if (!error)
-      break;
-  }
-
-  if (!error)
-  {
-    /* for now, compute the standard width and height from the `o' */
-    ta_latin_metrics_init_widths(metrics, face, 'o');
+    ta_latin_metrics_init_widths(metrics, face);
     ta_latin_metrics_init_blues(metrics, face);
     ta_latin_metrics_check_digits(metrics, face);
   }
@@ -553,9 +605,12 @@ ta_latin_metrics_scale_dim(TA_LatinMetrics metrics,
   FT_Fixed scale;
   FT_Pos delta;
   TA_LatinAxis axis;
+  FT_UInt ppem;
   FT_UInt nn;
 
 
+  ppem = metrics->root.scaler.face->size->metrics.x_ppem;
+
   if (dim == TA_DIMENSION_HORZ)
   {
     scale = scaler->x_scale;
@@ -577,6 +632,10 @@ ta_latin_metrics_scale_dim(TA_LatinMetrics metrics,
 
   /* correct X and Y scale to optimize the alignment of the top of */
   /* small letters to the pixel grid */
+  /* (if we do x-height snapping for this ppem value) */
+  if (!number_set_is_element(
+        metrics->root.globals->font->x_height_snapping_exceptions,
+        ppem))
   {
     TA_LatinAxis Axis = &metrics->axis[TA_DIMENSION_VERT];
     TA_LatinBlue blue = NULL;
@@ -597,11 +656,9 @@ ta_latin_metrics_scale_dim(TA_LatinMetrics metrics,
       FT_Pos threshold;
       FT_Pos fitted;
       FT_UInt limit;
-      FT_UInt ppem;
 
 
       scaled = FT_MulFix(blue->shoot.org, scaler->y_scale);
-      ppem = metrics->root.scaler.face->size->metrics.x_ppem;
       limit = metrics->root.globals->increase_x_height;
       threshold = 40;
 
@@ -722,6 +779,26 @@ ta_latin_metrics_scale_dim(TA_LatinMetrics metrics,
         blue->flags |= TA_LATIN_BLUE_ACTIVE;
       }
     }
+
+    /* the last two artificial blue zones are to be scaled */
+    /* with uncorrected scaling values */
+    {
+      TA_LatinAxis a = &metrics->axis[TA_DIMENSION_VERT];
+      TA_LatinBlue b;
+
+
+      b = &a->blues[a->blue_count];
+      b->ref.cur =
+      b->ref.fit =
+      b->shoot.cur =
+      b->shoot.fit = FT_MulFix(b->ref.org, a->org_scale) + delta;
+
+      b = &a->blues[a->blue_count + 1];
+      b->ref.cur =
+      b->ref.fit =
+      b->shoot.cur =
+      b->shoot.fit = FT_MulFix(b->ref.org, a->org_scale) + delta;
+    }
   }
 }
 
@@ -1388,7 +1465,7 @@ ta_latin_hints_compute_blue_edges(TA_GlyphHints hints,
   /* for each horizontal edge search the blue zone which is closest */
   for (; edge < edge_limit; edge++)
   {
-    FT_Int bb;
+    FT_UInt bb;
     TA_Width best_blue = NULL;
     FT_Pos best_dist; /* initial threshold */
 
@@ -1404,7 +1481,13 @@ ta_latin_hints_compute_blue_edges(TA_GlyphHints hints,
     if (best_dist > 64 / 2)
       best_dist = 64 / 2;
 
-    for (bb = 0; bb < TA_LATIN_BLUE_MAX; bb++)
+    /* this loop also handles the two extra blue zones */
+    /* for usWinAscent and usWinDescent */
+    /* if option `windows-compatibility' is set */
+    for (bb = 0;
+         bb < latin->blue_count
+              + (metrics->root.globals->font->windows_compatibility ? 2 : 0);
+         bb++)
     {
       TA_LatinBlue blue = latin->blues + bb;
       FT_Bool is_top_blue, is_major_dir;
@@ -1818,6 +1901,9 @@ ta_latin_hint_edges(TA_GlyphHints hints,
   TA_Edge anchor = NULL;
   FT_Int has_serifs = 0;
 
+#ifdef TA_DEBUG
+  FT_UInt num_actions = 0;
+#endif
 
   TA_LOG(("%s edge hinting\n", dim == TA_DIMENSION_VERT ? "horizontal"
                                                         : "vertical"));
@@ -1855,6 +1941,7 @@ ta_latin_hint_edges(TA_GlyphHints hints,
       if (!edge1)
         continue;
 
+#ifdef TA_DEBUG
       if (!anchor)
         TA_LOG(("  BLUE_ANCHOR: edge %d (opos=%.2f) snapped to %.2f,"
                   " was %.2f (anchor=edge %d)\n",
@@ -1865,6 +1952,9 @@ ta_latin_hint_edges(TA_GlyphHints hints,
                 edge1 - edges, edge1->opos / 64.0, blue->fit / 64.0,
                 edge1->pos / 64.0));
 
+      num_actions++;
+#endif
+
       edge1->pos = blue->fit;
       edge1->flags |= TA_EDGE_DONE;
 
@@ -1882,6 +1972,10 @@ ta_latin_hint_edges(TA_GlyphHints hints,
       {
         ta_latin_align_linked_edge(hints, dim, edge1, edge2);
         edge2->flags |= TA_EDGE_DONE;
+
+#ifdef TA_DEBUG
+        num_actions++;
+#endif
       }
 
       if (!anchor)
@@ -1916,6 +2010,10 @@ ta_latin_hint_edges(TA_GlyphHints hints,
 
       ta_latin_align_linked_edge(hints, dim, edge2, edge);
       edge->flags |= TA_EDGE_DONE;
+
+#ifdef TA_DEBUG
+      num_actions++;
+#endif
       continue;
     }
 
@@ -1985,6 +2083,10 @@ ta_latin_hint_edges(TA_GlyphHints hints,
                         edge, edge2, NULL, NULL, NULL);
 
       ta_latin_align_linked_edge(hints, dim, edge, edge2);
+
+#ifdef TA_DEBUG
+      num_actions += 2;
+#endif
     }
     else
     {
@@ -2114,15 +2216,23 @@ ta_latin_hint_edges(TA_GlyphHints hints,
         }
       }
 
+#ifdef TA_DEBUG
+      num_actions++;
+#endif
+
       edge->flags |= TA_EDGE_DONE;
       edge2->flags |= TA_EDGE_DONE;
 
       if (edge > edges
           && edge->pos < edge[-1].pos)
       {
+#ifdef TA_DEBUG
         TA_LOG(("  BOUND: edge %d (pos=%.2f) moved to %.2f\n",
                 edge - edges, edge->pos / 64.0, edge[-1].pos / 64.0));
 
+        num_actions++;
+#endif
+
         edge->pos = edge[-1].pos;
 
         if (hints->recorder)
@@ -2289,7 +2399,6 @@ ta_latin_hint_edges(TA_GlyphHints hints,
         else
         {
           edge->pos = anchor->pos + ((edge->opos - anchor->opos + 16) & ~31);
-
           TA_LOG(("  SERIF_LINK2: edge %d (opos=%.2f) snapped to %.2f\n",
                   edge - edges, edge->opos / 64.0, edge->pos / 64.0));
 
@@ -2299,13 +2408,19 @@ ta_latin_hint_edges(TA_GlyphHints hints,
         }
       }
 
+#ifdef TA_DEBUG
+      num_actions++;
+#endif
       edge->flags |= TA_EDGE_DONE;
 
       if (edge > edges
           && edge->pos < edge[-1].pos)
       {
+#ifdef TA_DEBUG
         TA_LOG(("  BOUND: edge %d (pos=%.2f) moved to %.2f\n",
                 edge - edges, edge->pos / 64.0, edge[-1].pos / 64.0));
+        num_actions++;
+#endif
 
         edge->pos = edge[-1].pos;
 
@@ -2318,9 +2433,13 @@ ta_latin_hint_edges(TA_GlyphHints hints,
           && edge[1].flags & TA_EDGE_DONE
           && edge->pos > edge[1].pos)
       {
+#ifdef TA_DEBUG
         TA_LOG(("  BOUND: edge %d (pos=%.2f) moved to %.2f\n",
                 edge - edges, edge->pos / 64.0, edge[1].pos / 64.0));
 
+        num_actions++;
+#endif
+
         edge->pos = edge[1].pos;
 
         if (hints->recorder)
@@ -2330,7 +2449,11 @@ ta_latin_hint_edges(TA_GlyphHints hints,
     }
   }
 
+#ifdef TA_DEBUG
+  if (!num_actions)
+    TA_LOG(("  (none)\n"));
   TA_LOG(("\n"));
+#endif
 }
 
 
@@ -2447,6 +2570,7 @@ const TA_ScriptClassRec ta_latin_script_class =
 {
   TA_SCRIPT_LATIN,
   ta_latin_uniranges,
+  'o',
 
   sizeof (TA_LatinMetricsRec),
 
