@@ -26,10 +26,25 @@
 #define COMPARE(str) \
           (len == (sizeof (str) - 1) \
            && !strncmp(start, str, sizeof (str) - 1))
-#define DUMP(str, arg) \
+#define DUMPVAL(str, arg) \
           fprintf(stderr, "%33s = %ld\n", \
                           (str), \
                           (FT_Long)(arg))
+#define DUMPSTR(str, arg) \
+          fprintf(stderr, "%33s = %s\n", \
+                          (str), \
+                          (arg))
+
+
+void
+TA_sfnt_set_properties(SFNT* sfnt,
+                       FONT* font)
+{
+  TA_FaceGlobals  globals = (TA_FaceGlobals)sfnt->face->autohint.data;
+
+
+  globals->increase_x_height = font->increase_x_height;
+}
 
 
 TA_Error
@@ -57,6 +72,9 @@ TTF_autohint(const char* options,
   FT_Long hinting_limit = -1;
   FT_Long increase_x_height = -1;
 
+  const char* x_height_snapping_exceptions_string = NULL;
+  number_range* x_height_snapping_exceptions = NULL;
+
   FT_Bool gray_strong_stem_width = 0;
   FT_Bool gdi_cleartype_strong_stem_width = 1;
   FT_Bool dw_cleartype_strong_stem_width = 0;
@@ -66,6 +84,7 @@ TTF_autohint(const char* options,
   TA_Info_Func info;
   void* info_data;
 
+  FT_Bool windows_compatibility = 0;
   FT_Bool ignore_restrictions = 0;
   FT_Bool pre_hinting = 0;
   FT_Bool hint_with_components = 0;
@@ -181,16 +200,16 @@ TTF_autohint(const char* options,
       progress_data = va_arg(ap, void*);
     else if (COMPARE("symbol"))
       symbol = (FT_Bool)va_arg(ap, FT_Int);
+    else if (COMPARE("windows-compatibility"))
+      windows_compatibility = (FT_Bool)va_arg(ap, FT_Int);
+    else if (COMPARE("x-height-snapping-exceptions"))
+      x_height_snapping_exceptions_string = va_arg(ap, const char*);
     else
     {
       error = TA_Err_Unknown_Argument;
       goto Err1;
     }
 
-    /*
-      x-height-snapping-exceptions
-     */
-
   End:
     if (!*op)
       break;
@@ -247,7 +266,8 @@ TTF_autohint(const char* options,
   if (hinting_limit < 0)
     hinting_limit = TA_HINTING_LIMIT;
 
-  if (increase_x_height > 0 && increase_x_height < 6)
+  if (increase_x_height > 0
+      && increase_x_height < TA_PROP_INCREASE_X_HEIGHT_MIN)
   {
     error = FT_Err_Invalid_Argument;
     goto Err1;
@@ -255,10 +275,24 @@ TTF_autohint(const char* options,
   if (increase_x_height < 0)
     increase_x_height = TA_INCREASE_X_HEIGHT;
 
+  if (x_height_snapping_exceptions_string)
+  {
+    const char* s = number_set_parse(x_height_snapping_exceptions_string,
+                                     &x_height_snapping_exceptions,
+                                     TA_PROP_INCREASE_X_HEIGHT_MIN,
+                                     0x7FFF);
+    if (*s)
+    {
+      error = FT_Err_Invalid_Argument;
+      goto Err1;
+    }
+  }
+
   font->hinting_range_min = (FT_UInt)hinting_range_min;
   font->hinting_range_max = (FT_UInt)hinting_range_max;
   font->hinting_limit = (FT_UInt)hinting_limit;
   font->increase_x_height = increase_x_height;
+  font->x_height_snapping_exceptions = x_height_snapping_exceptions;
 
   font->gray_strong_stem_width = gray_strong_stem_width;
   font->gdi_cleartype_strong_stem_width = gdi_cleartype_strong_stem_width;
@@ -269,6 +303,7 @@ TTF_autohint(const char* options,
   font->info = info;
   font->info_data = info_data;
 
+  font->windows_compatibility = windows_compatibility;
   font->ignore_restrictions = ignore_restrictions;
   font->pre_hinting = pre_hinting;
   font->hint_with_components = hint_with_components;
@@ -282,33 +317,43 @@ TTF_autohint(const char* options,
   /* dump parameters */
   if (debug)
   {
+    char *s;
+
+
     fprintf(stderr, "TTF_autohint parameters\n"
                     "=======================\n\n");
 
-    DUMP("dw-cleartype-strong-stem-width",
-         font->dw_cleartype_strong_stem_width);
-    DUMP("fallback-script",
-         font->fallback_script);
-    DUMP("gdi-cleartype-strong-stem-width",
-         font->gdi_cleartype_strong_stem_width);
-    DUMP("gray-strong-stem-width",
-         font->gray_strong_stem_width);
-    DUMP("hinting-limit",
-         font->hinting_limit);
-    DUMP("hinting-range-max",
-         font->hinting_range_max);
-    DUMP("hinting-range-min",
-         font->hinting_range_min);
-    DUMP("hint-with-components",
-         font->hint_with_components);
-    DUMP("ignore-restrictions",
-         font->ignore_restrictions);
-    DUMP("increase-x-height",
-         font->increase_x_height);
-    DUMP("pre-hinting",
-         font->pre_hinting);
-    DUMP("symbol",
-         font->symbol);
+    DUMPVAL("dw-cleartype-strong-stem-width",
+            font->dw_cleartype_strong_stem_width);
+    DUMPVAL("fallback-script",
+            font->fallback_script);
+    DUMPVAL("gdi-cleartype-strong-stem-width",
+            font->gdi_cleartype_strong_stem_width);
+    DUMPVAL("gray-strong-stem-width",
+            font->gray_strong_stem_width);
+    DUMPVAL("hinting-limit",
+            font->hinting_limit);
+    DUMPVAL("hinting-range-max",
+            font->hinting_range_max);
+    DUMPVAL("hinting-range-min",
+            font->hinting_range_min);
+    DUMPVAL("hint-with-components",
+            font->hint_with_components);
+    DUMPVAL("ignore-restrictions",
+            font->ignore_restrictions);
+    DUMPVAL("increase-x-height",
+            font->increase_x_height);
+    DUMPVAL("pre-hinting",
+            font->pre_hinting);
+    DUMPVAL("symbol",
+            font->symbol);
+    DUMPVAL("windows-compatibility",
+            font->windows_compatibility);
+
+    s = number_set_show(font->x_height_snapping_exceptions,
+                        TA_PROP_INCREASE_X_HEIGHT_MIN, 0x7FFF);
+    DUMPSTR("x-height-snapping-exceptions", s);
+    free(s);
 
     fprintf(stderr, "\n");
   }
@@ -373,10 +418,14 @@ TTF_autohint(const char* options,
     if (error)
       goto Err;
 
+    /* this call creates a `globals' object... */
     error = TA_sfnt_handle_coverage(sfnt, font);
     if (error)
       goto Err;
 
+    /* ... so that we now can initialize its properties */
+    TA_sfnt_set_properties(sfnt, font);
+
     /* check permission */
     if (sfnt->OS2_idx != MISSING)
     {
